#!/usr/bin/perl

#    Copyright (C) 2010  Christian Arnold
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    Christian Arnold <arnold@schlittermann.de>

use 5.010;
use strict;
use warnings;
use File::Basename;
use Pod::Usage;
use Getopt::Long;
use LWP::Simple;
use HTTP::Status;
use File::Path;
use File::Temp;
use if $ENV{DEBUG} => "Smart::Comments";

use lib "/usr/lib/nagios/plugins";
use utils qw (%ERRORS);

$ENV{LANG} = "POSIX";

my $ME      = basename $0;
my $VERSION = "0.4";

sub download($);
sub verify($);
sub execute($);
sub version($$);

my $opt = {
    url    => undef,
    dir    => "/var/tmp/nagios/check_exec",
    binary => "/usr/bin/gpg"
};

MAIN: {
    Getopt::Long::Configure('bundling');
    GetOptions(
        "u|url=s"    => \$opt->{url},
        "b|binary=s" => \$opt->{binary},
        "d|dir=s"    => \$opt->{dir},
        "h|help" => sub { pod2usage(-verbose => 1, -exitval => $ERRORS{OK}) },
        "m|man" => sub { pod2usage(-verbose => 2, -exitval => $ERRORS{OK}) },
        "V|version" => sub { version($ME, $VERSION); exit $ERRORS{OK}; }
    ) or pod2usage(-verbose => 1, -exitval => $ERRORS{CRITICAL});

    $opt->{url} // pod2usage(-verbose => 1, -exitval => $ERRORS{CRITICAL});

    # prepare the nest
    {
        my $err;
        mkpath($opt->{dir}, { mode => 0700, error => \$err });
        if (@$err) {
            for my $diag (@$err) {
                my ($directory, $message) = %$diag;
                say
                  "EXEC CRITICAL: Can't create directory $directory: $message";
            }
            exit $ERRORS{CRITICAL};
        }

        # files older than 1 hour can be removed
        unlink grep { -M > 1 / 24 } glob("$opt->{dir}/*");
    }

    # these two functions return File::Temp Objects!
    my $local = download($opt->{url});
    my $exe   = verify($local->filename);
    execute($exe->filename);
}

sub execute($) {
    my $run_file = shift;
    chmod 0500, $run_file or do {
        say "EXEC CRITICAL: Can't chmod $run_file ($!)";
        exit $ERRORS{CRITICAL};
    };

    { exec $run_file $ME };
    say "EXEC CRITICAL: Couldn't exec $run_file ($!)";
    exit $ERRORS{CRITICAL};
}

sub download($) {
    my $url = shift;
    my $local = File::Temp->new(DIR => $opt->{dir});

    my $rc = getstore($url, $local->filename);
    if (is_error($rc)) {

        if ($rc == 404) {
            say "EXEC OK: $url ", status_message($rc);
            exit $ERRORS{OK};
        }

        say "EXEC CRITICAL: $url ", status_message($rc);
        exit $ERRORS{CRITICAL};
    }

    return $local;
}

sub verify($) {
    my $local = shift;

    my $home = (getpwuid($>))[7];
    my $verified = File::Temp->new(DIR => $opt->{dir});

    my @r = qx($opt->{binary} --homedir $home/.gnupg --batch --yes --output $verified $local 2>&1);

    if ($?) {
        print "EXEC CRITICAL: @r";
        exit $ERRORS{CRITICAL};
    }

    $verified->close();    # if not closed, it stays busy
    return $verified;
}

sub version($$) {
    my $progname = shift;
    my $version  = shift;

    print <<_VERSION;
$progname version $version
Copyright (C) 2010 by Christian Arnold and Schlittermann internet & unix support.

$ME comes with ABSOLUTELY NO WARRANTY. This is free software,
and you are welcome to redistribute it under certain conditions.
See the GNU General Public Licence for details.
_VERSION
}

__END__

=head1 NAME

check_exec - nagios plugin to download/verify/execute a program file

=head1 SYNOPSIS

check_exec -u|--url
          [-b|--binary path]
          [-d|--dir dir]

check_exec [-h|--help]
check_exec [-m|--man]
check_exec [-v|--version]

=head1 OPTIONS

=over

=item B<-u>|B<--url> I<url>

Download url for generic script.

=item B<-b>|B<--binary> I<path>

Path to gpg binary program (default: /usr/bin/gpg)

=item B<-d>|B<--dir> I<dir>

Location for store download script (default: /var/tmp/nagios)

=item B<-h>|B<--help>

Print detailed help screen.

=item B<-m>|B<--man>

Print manual page.

=item B<-V>|B<--version>

Print version information.

=back

=head1 DESCRIPTION

This plugin loads a program file via http or https from a
server and verifies its validity based on a gpg key.

=head1 VERSION

This man page is current for version 0.4 of check_exec.

=head1 AUTHOR

Written by Christian Arnold L<arnold@schlittermann.de>

=head1 COPYRIGHT

Copyright (C) 2010 by Christian Arnold and Schlittermann internet & unix support.
This is free software, and you are welcome to redistribute it under certain conditions.
See the GNU General Public Licence for details.

=cut
